import _extends from "@babel/runtime/helpers/esm/extends";
import { createElement as h, useState, useEffect } from 'react';
import PropTypes from 'prop-types';
import { routes, components } from './internal/PropTypes';
import { fromReactTreeToJSON } from './internal/ReactUtils';
import { createRouterHistory, createRouterObject } from './internal/RouterUtils';
import { createTransitionManager } from './internal/createTransitionManager';
import RouterContext from './internal/Context';
import Route from './Route';
import { jsx as _jsx } from "react/jsx-runtime";

var renderRouterContent = function renderRouterContent(_ref) {
  var components = _ref.components,
      params = _ref.params,
      router = _ref.router;
  var location = router.location,
      routes = router.routes; // get the latest matched route

  var route = routes && routes.length ? routes[routes.length - 1] : [];
  var routerInfo = {
    location: location,
    params: params,
    routeParams: params,
    route: route,
    router: router,
    routes: routes
  };
  return components.reduceRight(function (children, component) {
    return /*#__PURE__*/h(component, routerInfo, children);
  }, null);
};

var createRoutes = function createRoutes(_ref2) {
  var children = _ref2.children,
      routes = _ref2.routes;
  if (routes) return fromReactTreeToJSON(routes);
  return fromReactTreeToJSON( /*#__PURE__*/h(Route, null, children));
};

var Router = function Router(_ref3) {
  var _matchContext$transit;

  var children = _ref3.children,
      _ref3$components = _ref3.components,
      components = _ref3$components === void 0 ? [] : _ref3$components,
      _ref3$history = _ref3.history,
      history = _ref3$history === void 0 ? createRouterHistory() : _ref3$history,
      matchContext = _ref3.matchContext,
      onError = _ref3.onError,
      params = _ref3.params,
      routerFromProps = _ref3.router,
      routes = _ref3.routes;
  // we might be using Router with match, if it's the case
  // when we should have from props the transitionManager and the router object
  // if not, we are going to create it with the needed info
  var transitionManager = (_matchContext$transit = matchContext == null ? void 0 : matchContext.transitionManager) != null ? _matchContext$transit : createTransitionManager({
    history: history,
    jsonRoutes: createRoutes({
      children: children,
      routes: routes
    })
  });
  var router = routerFromProps != null ? routerFromProps : createRouterObject(history, transitionManager.isActive);

  var _useState = useState({
    router: router,
    params: params,
    components: components
  }),
      state = _useState[0],
      setState = _useState[1];

  useEffect(function () {
    var prevState = {};

    var handleTransition = function handleTransition(err, nextState) {
      if (err) {
        if (onError) return onError(err);
        throw err;
      } // avoid not needed re-renders of the state if the prevState and the nextState
      // are the same reference


      if (prevState === nextState) return;
      prevState = nextState;
      var components = nextState.components,
          params = nextState.params,
          location = nextState.location,
          routes = nextState.routes;

      var nextRouter = _extends({}, state.router, {
        params: params,
        location: location,
        routes: routes
      });

      setState({
        router: nextRouter,
        params: params,
        components: components
      });
    };

    var unlisten = transitionManager.listen(handleTransition);
    return function () {
      return unlisten();
    };
  }, []); // eslint-disable-line

  return /*#__PURE__*/_jsx(RouterContext.Provider, {
    value: state,
    children: renderRouterContent(state)
  });
};

Router.displayName = 'Router';
export default Router;